#include "stdafx.h"

#include "distance.h"
#include <cmath>
//#include <iomanip>
using namespace std;
//extern map<string, int> calls;


/*In the original setup:

double distance2norm(const PVideoFrame &from, const PVideoFrame &to)

double CacheDistance::distance(int a, int b, IScriptEnvironment *env) //const
{
       intpair pair(min(a,b), max(a,b));

       //return the value if it's cached
       map<intpair, double>::const_iterator i = cache.find(pair);
       if(i != cache.end())
               return i->second;

       //otherwise, compute it and cache it
       PVideoFrame framea = clip->GetFrame(a, env);
       PVideoFrame frameb = clip->GetFrame(b, env);
       double result = distance2norm(framea, frameb);
       cache[pair] = result;
       return result;
}

... the class CacheDistance is misnamed.
It both computes and caches the distance and as in all such cases the
caching is
the secondary function. Any filter can cache its results; and caching is an
implementation issue which higher layer should not be aware of.Indeed,
at higher
levels the expression CacheDistance(clip) looks anomalous and
ComputeDistance(clip) appropriate. In the few cases were caching needs to be
controlled at higher layers, the caching should be in a separate object,as in
Cache(ComputeDistance(clip)).
Although the function distance2norm does most of the work, it is still not the
place of distance computation, for it has no awareness of the clip or of frame
numbers. This becomes clearer when the computation logic becomes more
complicated, for example when two distances are computed at once (for
efficiency reasons)... the change must be made in CacheDistance/ComputeDistance
rather than in the function.*/


ComputeDistance::ComputeDistance(/*const*/ PClip _clip) : clip(_clip), 
cache_d1(clip->GetVideoInfo().num_frames),
cache_d2(clip->GetVideoInfo().num_frames)
{
}

void ComputeDistance::computepair(int n, IScriptEnvironment *env) /*const*/ //computes d(n,n+1)
{
	PVideoFrame frame0 = clip->GetFrame(n, env);
	PVideoFrame frame1 = clip->GetFrame(n+1, env);
	//double result = distance2norm(frame0, frame1);
	//cache[make_pair(n, n+1)] = result;
	cache_d1[n].set(  distance2norm(frame0, frame1)  );
}
void ComputeDistance::computetriad(int n, IScriptEnvironment *env) /*const*/ //computes d(n,n+1), d(n,n+2)
{
	PVideoFrame frame0 = clip->GetFrame(n, env);
	PVideoFrame frame1 = clip->GetFrame(n+1, env);
	PVideoFrame frame2 = clip->GetFrame(n+2, env);
	bool optimisable = ((env->GetCPUFlags() & CPUF_SSE2) != 0);
	pair<double, double> result = (optimisable ? doubledistance_SSE2 : doubledistance_C)(frame0, frame1, frame2);
	//if(result.first != distance2norm(frame0, frame1) || result.second != distance2norm(frame0, frame2))
	//	env->ThrowError("ReverseBlend: Anomaly in triple distance calculation.");;
	//cache[make_pair(n, n+1)] = result.first;
	//cache[make_pair(n, n+2)] = result.second;
	cache_d1[n].set(  result.first  );
	cache_d2[n].set(  result.second  );
}


double ComputeDistance::distance(int a, int b, IScriptEnvironment *env) /*const*/
{/*++calls["ComputeDistance::distance"];*/
	int n = min(a,b);
	int separation = max(a,b) - n;
	if(separation == 1)
	{
		if(!cache_d1[n].found)
		{
			if(n+2 < clip->GetVideoInfo().num_frames)
				computetriad(n, env);//computes d(n,n+1), d(n,n+2)
			else if(n-1 >= 0)
				computetriad(n-1, env);//computes d(n-1,n), d(n-1,n+1)
			else
				computepair(n, env);
		}
		return cache_d1[n].d;
	}
	else if(separation == 2)
	{
		if(!cache_d2[n].found)
			computetriad(n, env);//computes d(n,n+1), d(n,n+2)
		return cache_d2[n].d;
	}
	else
		env->ThrowError("ReverseBlend: Invalid distance requested.");
	throw;
}

int ComputeDistance::frames() const
{
	return clip->GetVideoInfo().num_frames;
}

ostream &write(ostream &o, Distance &d, IScriptEnvironment *env)
{
	o << d.frames() << "\n";
	for(int i = 0; i < d.frames()-2; ++i)
		o << i << "\t" << d.distance(i, i+1, env) << "\t" << d.distance(i, i+2, env) << "\n";
	o << d.frames()-2 << "\t" << d.distance(d.frames()-2, d.frames()-1, env) << "\n";
	return o;
}

ReadDistance::ReadDistance(istream &input)
{
	input >> _frames;// >> "\n";
	d1.resize(_frames-1);
	d2.resize(_frames-2);
	int discard;
	for(int i = 0; i < _frames-2; ++i)
		input >> discard >> d1.at(i) >> d2.at(i);
	input >> discard >> d1.at(_frames-2);
}
//ofstream dlog("log.txt");
double ReadDistance::distance(int a, int b, IScriptEnvironment *env) /*const*/
{
	int m = min(a,b);
	int n = max(a,b);
	if(m < 0 || /*m >= _frames || n < 0 ||*/ n >= _frames)
		env->ThrowError("ReverseBlend: invalid arguments to ReadDistance::distance.");
	if(n == m + 1)
	{
		//dlog << "d(" << n  << ", " << m << ") = " << d1.at(m) << "\n";
		return d1.at(m);
	}
	if(n == m + 2)
	{
		//dlog << "d(" << n  << ", " << m << ") = " << d1.at(m) << "\n";
		return d2.at(m);
	}
	env->ThrowError("ReverseBlend: arguments to ReadDistance::distance differ by invalid amount.");
		throw;//to prevent compiler warning
}

int ReadDistance::frames() const
{
	return _frames;
}
//
//
//void Counter ::log (const char*string)
//{
//	ofstream output(string,ios);
//	;return _frames;
//}
//
