#include "stdafx.h"

#include "apply.h"
#include <fstream>
using namespace std;

ofstream log("log.txt");

double framerate(const PClip &clip)
{
	VideoInfo vi =clip->GetVideoInfo();
	return double(vi.fps_numerator)/vi.fps_denominator;
}

AVSValue __cdecl Create_RestoreFPS(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	PClip blended = args[0].AsClip();
	double originalFPS = double (args[1].AsFloat());
	double blendedFPS = framerate(blended);
	if (originalFPS >= blendedFPS) 
		env->ThrowError("Cannot restore frame rate higher than that of source.");
	double phase = double (args[2].AsFloat());
	Blend *blend = new Resample(originalFPS, blendedFPS, phase);
	return new ReverseBlend(blended, *blend, env);  
}

AVSValue __cdecl Create_IVTC(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	PClip blended = args[0].AsClip();
	double blendedFPS = framerate(blended);
	int skip = args[1].AsInt();
	Blend *blend = new Telecine(skip);
	return new ReverseBlend(blended, *blend, env);  
}

AVSValue __cdecl Create_RestoreFPSIVTC(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	PClip blended = args[0].AsClip();
	double originalFPS = double (args[1].AsFloat());
	double blendedFPS = framerate(blended);
	if (0.8*originalFPS >= blendedFPS) 
		env->ThrowError("Cannot restore frame rate higher than that of source.");
	double phase = double (args[2].AsFloat());
	Blend *blend2 = new Resample(originalFPS, blendedFPS, phase);

	int skip = args[3].AsInt();
	Blend *blend1 = new Telecine(skip);

	Blend *composite = new Composition(*blend1, *blend2);

	return new ReverseBlend(blended, *composite, env);  
}

AVSValue __cdecl Create_TelecineConvertFPS(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	PClip input = args[0].AsClip();
	Blend *blend1 = new Telecine(0);
	Blend *blend2 = new Resample(framerate(input)*1.25, double (args[1].AsFloat()), 0.0);
	Blend *composite = new Composition(*blend1, *blend2);

	return new ForwardBlend(input, *composite, env);  
}


AVSValue __cdecl Create_ThresholdIncreaseFPS(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	PClip input = args[0].AsClip();
	double originalFPS = framerate(input);
	double blendedFPS = double (args[1].AsFloat());
	double threshold = double (args[2].AsFloat());
	Blend *blend = new ThresholdUpsample(originalFPS, blendedFPS, 0.0, threshold);
	return new ForwardBlend(input, *blend, env);  
}


AVSValue __cdecl Create_Reblend(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	PClip input = args[0].AsClip();
	double lambda = double (args[1].AsFloat());
	Blend *blend = new Reblend(lambda);

	return new ForwardBlend(input, *blend, env);  
}

AVSValue __cdecl Create_Test(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	PClip output = args[0].AsClip();
	/*Blend *blend3 = new Resample(Reverse(), vi.num_frames, 29.97, 25, 0);
	Blend *blend2 = new Telecine(Reverse(), blend3->inframes(), 29.97, 0);
	Blend *blend1 = new Duplicate(Reverse(), blend2->inframes(), 23.976);
	Blend *composite23 = new Composition(*blend2, *blend3);
	Blend *composite = new Composition(*blend1, *composite23);*/

	Blend *blend2 = new Resample(30, 25, 0);
	//Blend *blend2 = new Resample(Reverse(), vi.num_frames, 15, 16, 0);
	Blend *blend1 = new Telecine(0);
	Blend *composite = new Composition(*blend1, *blend2);
	Blend *test = new TestBlend();


//for (int i=0;i<21;++i) 
//log<<i << "\t" <<blend1-> weight(i)<< "\n";
//log<< "\n";
//
for (int i=0;i<20;++i) 
log<<i << "\t" <<blend2-> weight(i)<< "\n";
log<< "\n";

//for (int i=0;i<30;++i) 
//log<<i << "\t" <<test-> weight(i)<< "\n";
//log<< "\n";

for (int i=0;i<211;++i) 
log<<i << "\t" <<composite-> weight(i)<< "\n";
log<< "\n";


	log.flush();

//	env->ThrowError("In test mode.");
	return new ReverseBlend(output, *test , env);  
}

extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env) 
{
	env->AddFunction("RestoreFPS", "cff", Create_RestoreFPS, 0);
//	env->AddFunction("IVTC", "ci", Create_IVTC, 0);
	env->AddFunction("RestoreFPSIVTC", "cffi", Create_RestoreFPSIVTC, 0);
	env->AddFunction("TelecineConvertFPS", "cf", Create_TelecineConvertFPS, 0);
	env->AddFunction("ThresholdIncreaseFPS", "cff", Create_ThresholdIncreaseFPS, 0);
	env->AddFunction("Reblend", "cf", Create_Reblend, 0);
	env->AddFunction("Test", "c", Create_Test, 0);

	return "ReverseBlend plugin";
}
