#include "stdafx.h"
#include "typed.h"

#include <cmath>

SelectPt And::GetPt(int n, IScriptEnvironment* env)
{
		SelectPt pt1 = first.point( n, env );
		SelectPt pt2 = second.point( n, env );
		return SelectPt (pt1.selected && pt2.selected);
}
AVSValue __cdecl Create_And(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	return new And(args[0].AsClip(), Select(args[0].AsClip()), Select(args[1].AsClip()), env);  
}


SelectPt Or::GetPt(int n, IScriptEnvironment* env)
{
		SelectPt pt1 = first.point( n, env );
		SelectPt pt2 = second.point( n, env );
		return SelectPt (pt1.selected || pt2.selected);
}
AVSValue __cdecl Create_Or(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	return new Or(args[0].AsClip(), Select(args[0].AsClip()), Select(args[1].AsClip()), env);  
}


WriteNumber::WriteNumber(PClip _child, Number match, const char *filename, IScriptEnvironment* env) : GenericVideoFilter(_child) 
{
	ofstream output (filename);
	for (int i = 0; i < vi.num_frames; ++i)
	{
		NumberPt pt = match.point( i, env );
		output << i << "\t" << pt.value << "\n";
		output.flush();
	}
}
AVSValue __cdecl Create_WriteNumber(AVSValue args, void* user_data, IScriptEnvironment* env)
	{ return new WriteNumber(args[0].AsClip(),Number(args[0].AsClip()), args[1].AsString("number.txt"), env); }

SelectPt IsAbove::GetPt(int n, IScriptEnvironment* env)
{
		NumberPt pt = number.point( n, env );
		return SelectPt (pt.value> threshold);
}
AVSValue __cdecl Create_IsAbove(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	return new IsAbove(args[0].AsClip(), Number(args[0].AsClip()), args[1].AsFloat(), env);  
}


SelectPt IsBelow::GetPt(int n, IScriptEnvironment* env)
{
		NumberPt pt = number.point( n, env );
		return SelectPt (pt.value< threshold);
}
AVSValue __cdecl Create_IsBelow(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	return new IsBelow(args[0].AsClip(), Number(args[0].AsClip()), args[1].AsFloat(), env);  
}

SelectPt IsEqualTo::GetPt(int n, IScriptEnvironment* env)
{
		NumberPt pt = number.point( n, env );
		return SelectPt (pt.value==threshold);
}
AVSValue __cdecl Create_IsEqualTo(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	return new IsEqualTo(args[0].AsClip(), Number(args[0].AsClip()), args[1].AsFloat(), env);  
}


NumberPt Minimum::GetPt(int n, IScriptEnvironment* env)
{
	double result = arguments[0].point( n, env ).value;
	for (unsigned int i = 1; i < arguments.size(); ++i)
		result = min(result, arguments[i].point( n, env ).value);

	return result;
}
AVSValue __cdecl Create_Minimum(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	AVSValue array = args[0];
	int noarguments = array.ArraySize();

	vector<Number> arguments;
	for (int i=0; i < noarguments; ++i)
		arguments.push_back(  Number(array[i].AsClip())  );

	return new Minimum(array[0].AsClip(), arguments, env); 
}



NumberPt Maximum::GetPt(int n, IScriptEnvironment* env)
{
	double result = arguments[0].point( n, env ).value;
	for (unsigned int i = 1; i < arguments.size(); ++i)
		result = max(result, arguments[i].point( n, env ).value);

	return result;
}
AVSValue __cdecl Create_Maximum(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	AVSValue array = args[0];
	int noarguments = array.ArraySize();

	vector<Number> arguments;
	for (int i=0; i < noarguments; ++i)
		arguments.push_back(  Number(array[i].AsClip())  );

	return new Maximum(array[0].AsClip(), arguments, env); 
}




NumberPt BlendAngle::GetPt(int n, IScriptEnvironment* env)
{
	if(n-1 < 0 || n+1 >= distancecache.frames())
		return 0.0;
	double dp	= distancecache.distance(n-1, n, env);
	double dn	= distancecache.distance(n, n+1, env);
	double dpn	= distancecache.distance(n-1, n+1, env);

	//use the cosine rule to find the cosine of the angle (previous-this-next) in question
	double c = (dp*dp + dn*dn - dpn*dpn)/(2*dp*dn);

	return acos(c)*180.0/3.14159265;
}
AVSValue __cdecl Create_BlendAngle(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	return new BlendAngle(args[0].AsClip(), env);  
}


NumberPt BlendWeight::GetPt(int n, IScriptEnvironment* env)
{
	if(n-1 < 0 || n+1 >= distancecache.frames())
		return 0.5;
	double dp	= distancecache.distance(n-1, n, env);
	double dn	= distancecache.distance(n, n+1, env);
	double dpn	= distancecache.distance(n-1, n+1, env);

	//use the cosine rule to find the cosine of the angle (this-previous-next) in question
	double c = (dp*dp + dpn*dpn - dn*dn)/(2*dp*dpn);
	double projection = c*dp;//projection of previous-this onto previous-next
	return projection/dpn;
}
AVSValue __cdecl Create_BlendWeight(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	return new BlendWeight(args[0].AsClip(), env);  
}

